# -*- coding: utf-8 -*-

import sys
from radish.colorful import colorful


class RadishError(Exception):
    def __init__(self, msg):
        self._msg = msg

    def __str__(self):
        return colorful.bold_red(self._msg)

    def show(self):
        sys.stderr.write("%s %s\n" % (colorful.bold_red("radish error:"), self))


class FeatureFileNotValidError(RadishError):
    """Raised when a feature file could not be parsed"""
    def __init__(self, feature_file):
        self._feature_file = feature_file

    def __str__(self):
        return colorful.red("The feature file '%s' is not valid" % (self._feature_file))


class StepLoadingError(RadishError):
    """Raised when a step decorator regex is not valid to compile"""
    def __init__(self, regex):
        self._regex = regex

    def __str__(self):
        return colorful.red("The step decorator regex '%s' is not valid to compile" % (self._regex))


class StepDefinitionNotFoundError(RadishError):
    """Raised when a step definition could not be found"""
    def __init__(self, step_sentence, step_filename, step_line_no):
        self._step_sentence = step_sentence
        self._step_filename = step_filename
        self._step_line_no = step_line_no

    def show(self):
        import re
        sys.stderr.write(colorful.bold_red("%s:%d" % (self._step_filename, self._step_line_no)))
        sys.stderr.write(colorful.red(": error: no step definition found for '%s'\n" % (self._step_sentence)))
        sys.stderr.write("you might want to add the following to your steps.py:\n\n")
        sentence = self._step_sentence
        sentence = re.sub("^When ", "", sentence)
        sentence = re.sub("^Given ", "", sentence)
        sentence = re.sub("^Then ", "", sentence)
        sys.stderr.write("@step(u'%s')\n" % sentence)
        sentence = sentence.replace(" ", "_")
        sentence = sentence.replace(".", "_")
        sys.stderr.write("def %s(step):\n" % sentence)
        sys.stderr.write("    assert False, \"Not implemented yet\"\n\n")


class FeatureFileNotFoundError(RadishError):
    """Raised when a feature file could not be found"""
    def __init__(self, path):
        self._path = path

    def __str__(self):
        return colorful.red("The feature file '%s' could not be found" % (self._path))


class BasedirNotFoundError(RadishError):
    """Raised when the basedir could not be found"""
    def __init__(self, basedir):
        self._basedir = basedir

    def __str__(self):
        return colorful.red("basedir not found: '%s'" % (self._basedir))

    def show(self):
        RadishError.show(self)
        sys.stderr.write("\nyou might want to create the basedir and initial steps and terrain files by running radish -c\n\n")


class StepDefinitionFileNotFoundError(RadishError):
    """Raised when the step definition file could not be found"""
    def __init__(self, root, pattern):
        self._root = root
        self._pattern = pattern

    def __str__(self):
        return colorful.red("The step definition file '%s' could not be found in the directory '%s'" % (self._pattern, self._root))


class NoMetricUtilFoundError(RadishError):
    """Raised if metric should be shown but no utils hook was found"""
    def __str__(self):
        return colorful.red("Could not found util hook to show metric - implement: utils( \"show_metrics\" )")


class ValidationError(RadishError):
    """Raised when a validation error occured during run"""

    def __str__(self):
        return colorful.bold_red(self._msg)



# -*- coding: utf-8 -*-

from radish.singleton import singleton
from radish.step import Step


@singleton()
class HookRegistry(object):
    hooks = {
        "all": {
            "before": [],
            "after": []
        },
        "feature": {
            "before": [],
            "after": []
        },
        "scenario": {
            "before": [],
            "after": []
        },
        "step": {
            "before": [],
            "after": []
        }
    }

    possible_hooks = (("all", "all"), ("feature", "each_feature"), ("scenario", "each_scenario"), ("step", "each_step"))

    class Hooker(object):
        def __init__(self, when):
            self.when = when

        @classmethod
        def add_hook(cls, what, name):
            def wrapper(self, func):
                HookRegistry().register(self.when, what, func)
                return func
            wrapper.__name__ = wrapper.fn_name = name
            setattr(cls, name, wrapper)

    def register(self, when, what, func):
        self.hooks[what][when].append(func)

    def call_hook(self, when, what, *args, **kw):
        for h in self.hooks[what][when]:
            try:
                h(*args, **kw)
            except KeyboardInterrupt:
                continue
            except Exception, e:
                return Step.FailReason(e)
        return None

for what, name in HookRegistry.possible_hooks:
    HookRegistry.Hooker.add_hook(what, name)

before = HookRegistry.Hooker("before")
after = HookRegistry.Hooker("after")



# -*- coding: utf-8 -*-

from radish.config import Config
from radish.step import Step


class Scenario(object):
    def __init__(self, id, feature, sentence, filename, line_no):
        self._id = id
        self._feature = feature
        self._sentence = sentence
        self._filename = filename
        self._line_no = line_no
        self._steps = []

    def get_id(self):
        return self._id

    def set_id(self, id):
        self._id = id

    def get_feature(self):
        return self._feature

    def get_line_no(self):
        return self._line_no

    def get_sentence(self):
        return self._sentence

    def set_sentence(self, sentence):
        self._sentence = sentence

    def get_indentation(self):
        return "  " + " " * len(str(Config().highest_feature_id)) + "  "

    def get_duration(self):
        d = 0
        for s in self._steps:
            d += s.get_duration()
        return d

    def is_dry_run(self):
        return Config().dry_run

    def get_steps(self):
        return self._steps

    def has_passed(self):
        skipped = True
        for s in self._steps:
            if s.has_passed() is False:
                return False
            elif s.has_passed():
                skipped = False
        return None if skipped else True

    def append_step(self, step):
        if isinstance(step, Step):
            self._steps.append(step)



# -*- coding: utf-8 -*-

from radish.singleton import singleton


@singleton()
class UtilRegistry(object):
    def __init__(self):
        self._utils = {}

    def register(self, util, func):
        self._utils[util] = func

    def has_util(self, util):
        return util in self._utils

    def call_util(self, util, *args, **kw):
        if self.has_util(util):
            return self._utils[util](*args, **kw)
        return None


def utils(util):
    def _wrapper(func):
        UtilRegistry().register(util, func)
        return func
    return _wrapper



# -*- coding: utf-8 -*-


class Metrics(object):
    def __init__(self, features):
        self._features = features

    def calculate(self):
        metrics = {}
        for feature in self._features:
            for scenario in feature.get_scenarios():
                for step in scenario.get_steps():
                    if step.get_metric_indicators():
                        for indicator in step.get_metric_indicators():
                            try:
                                metrics[indicator] += 1
                            except KeyError:
                                metrics[indicator] = 1
        return metrics



# -*- coding: utf-8 -*-

import sys

import radish

try:
    from colorama import init as init_ansi_colors_on_windows
    init_ansi_colors_on_windows()
except ImportError:
    pass


class Modifiers:
    """ANSI modifiers for Colorful"""
    reset = 0
    bold = 1
    italic = 3
    underline = 4
    blink = 5
    inverse = 7
    strikethrough = 9


class ForeColors:
    """ANSI foreground colors for Colorful"""
    black = 30
    red = 31
    green = 32
    brown = 33
    blue = 34
    magenta = 35
    cyan = 36
    white = 37
    normal = 39


class BackColors:
    """ANSI background colors for Colorful"""
    black = 40
    red = 41
    green = 42
    yellow = 43
    blue = 44
    magenta = 45
    cyan = 46
    white = 47
    normal = 49


class ANSISequenceNotFoundError(Exception):
    """Error if an ANSI sequence could not be found"""
    def __init__(self, modetype, name):
        self._modetype = modetype
        self._name = name

    def __str__(self):
        return "ANSI Sequence `%s` could not be found in `%s`" % (self._name, self._modetype)


class ColorfulParser:
    """Colorful is a class to decorate text with ANSI colors and modifiers."""
    _modifiers = Modifiers()
    _forecolors = ForeColors()
    _backcolors = BackColors()

    _modifier_splitter = "and"
    _backcolor_splitter = "on"

    @classmethod
    def _is_modifier(cls, modifier):
        return hasattr(cls._modifiers, modifier)

    @classmethod
    def _get_ansi_decorator(cls, mode):
        return "\033[%sm" % (mode)

    @classmethod
    def _translate_to_ansi_decorator(cls, modetype, name):
        if hasattr(modetype, name):
            return cls._get_ansi_decorator(getattr(modetype, name))
        else:
            raise ANSISequenceNotFoundError(modetype.__class__.__name__, name)

    @classmethod
    def parse_attr(cls, attr):
        parts = attr.split("_")

        modifiers = ""
        forecolor = ""
        backcolor = ""

        if cls._is_modifier(parts[0]):
            modifiers += cls._translate_to_ansi_decorator(cls._modifiers, parts[0])
            parts = parts[1:]

        if cls._modifier_splitter in parts:
            for part in [parts[p + 1] if p + 1 < len(parts) else None for p in range(len(parts)) if parts[p] == cls._modifier_splitter]:
                modifiers += cls._translate_to_ansi_decorator(cls._modifiers, part)
            parts = parts[(len(parts) - parts[-1::-1].index(cls._modifier_splitter) - 1) + 2:]

        if "on" in parts:
            backcolor = cls._translate_to_ansi_decorator(cls._backcolors, parts[parts.index(cls._backcolor_splitter) + 1])
            parts = parts[:-2]

        if len(parts) > 0:
            forecolor = cls._translate_to_ansi_decorator(cls._forecolors, parts[0])

        return modifiers + forecolor + backcolor + "%s" + cls._translate_to_ansi_decorator(cls._modifiers, "reset")

    @classmethod
    def encode(cls, string):
        if isinstance(string, unicode):
            string = string.encode("utf-8")
        return string


class ColorfulMeta(type):
    colors = True

    class _ColorfulOut(type):
        def __getattr__(cls, attr):
            def decorated_text(text):
                output = text
                if not radish.Config().no_colors:
                    output = ColorfulParser.parse_attr(attr) % (text)

                print(ColorfulParser.encode(output))
                sys.stdout.flush()
            return decorated_text

    out = _ColorfulOut("out", (object, ), {})

    def __getattr__(cls, attr):
        def decorated_text(text):
            output = text
            if not radish.Config().no_colors:
                output = ColorfulParser.parse_attr(attr) % (text)
            return ColorfulParser.encode(output)
        return decorated_text

colorful = ColorfulMeta("cf", (object, ), {})



#!/usr/bin/python
# -*- coding: utf-8 -*-

import radish

import os
import sys
import time
import shutil

from docopt import docopt


def main():
    """
Usage:
    radish <features>...
           [-b=<basedir> | --basedir=<basedir>]
           [-m=<marker> | --marker=<marker>] [-p=<profile> | --profile=<profile>]
           [-d | --dry-run] [-a | --abort-fail]
           [-t | --with-traceback]
           [-x=<output> | --xunit-file=<output> [--split-xunit]]
           [--no-colors] [--no-line-jump] [--no-overwrite]
           [--no-indentation] [--no-duration] [--no-numbers]
           [--no-skipped-steps] [--with-section-names]
           [--show-metrics]
    radish (-c | --create-basedir)  [-b=<basedir> | --basedir=<basedir>]
    radish (-h | --help)
    radish (-v | --version)

Arguments:
    features                             feature files to run

Options:
    -h --help                            show this screen
    -v --version                         show version

    -c --create-basedir                  create basedir and initial steps and terrain files
    -b=<basedir> --basedir=<basedir>     set base dir from where the step.py and terrain.py will be loaded [default: $PWD/radish]
    -m=<marker> --marker=<marker>        specific marker which you can use to implement some kind of logging delimitiers [default: time.time()]
    -d --dry-run                         execute a dry run to validate steps
    -a --abort-fail                      abort run if one step fails
    -p=<profile> --profile=<profile>     define porfile which you can use in your step implementation
    -t --with-traceback                  print traceback if a step fails

    -x=<output> --xunit-file=<output>    generate xunit file after run at specific location
    --split-xunit                        split the xunit file into multiple files - one file per inputted feature file

    --no-colors                          do not print in colors
    --no-line-jump                       do not jump up lines to rewrite in another color
    --no-overwrite                       do not overwrite step
    --no-duration                        do not print duration after run
    --no-numbers                         do not print numbers before feature, scenario and step sentences
    --no-indentation                     do not print any indentation before sentences
    --no-skipped-steps                   do not print skipped steps
    --with-section-names                 print section name before feature and scenario sentences

    --show-metrics                       show metrics of given feature files after run

(C) Copyright 2013 by Timo Furrer <tuxtimo@gmail.com>"""

    arguments = docopt("radish %s\n%s" % (radish.version.__version__, main.__doc__), version=radish.version.__version__)

    cf = radish.Config()
    cf.no_colors = arguments["--no-colors"]

    if arguments['--create-basedir']:
        datadir = os.path.join(radish.__path__[0], "../templates")
        basedir = radish.FileSystemHelper.expand(arguments["--basedir"])
        if os.path.exists(basedir):
            print("basedir already exists: %s" % basedir)
        else:
            print "creating %s" % basedir
            os.mkdir(basedir)

        filename = "%s/steps.py" % basedir
        if os.path.exists(filename):
            print("file already exists: %s" % filename)
        else:
            print("creating %s" % filename)
            shutil.copyfile(os.path.join(datadir, "steps.py"), filename)

        filename = "%s/terrain.py" % basedir
        if os.path.exists(filename):
            print("file already exists: %s" % filename)
        else:
            print("creating %s" % filename)
            shutil.copyfile(os.path.join(datadir, "terrain.py"), filename)

        sys.exit(0)

    exitCode = 0
    try:
        # initialize config object
        # FIXME: clean up config and arguments
        cf.no_line_jump = arguments["--no-line-jump"]
        cf.SetBasedir(radish.FileSystemHelper.expand(os.path.join(os.getcwd(), "radish") if arguments["--basedir"] == "$PWD/radish/" else arguments["--basedir"]))
        cf.feature_files = arguments["<features>"]
        cf.abort_fail = arguments["--abort-fail"]
        cf.marker = time.time() if arguments["--marker"] == "time.time()" else arguments["--marker"]
        cf.create_basedir = arguments["--create-basedir"]
        cf.dry_run = arguments["--dry-run"]
        cf.xunit_file = arguments["--xunit-file"]
        cf.split_xunit = arguments["--split-xunit"]
        cf.profile = arguments["--profile"]
        cf.no_numbers = arguments["--no-numbers"]
        cf.no_indentation = arguments["--no-indentation"]
        cf.no_overwrite = arguments["--no-overwrite"]
        cf.no_duration = arguments["--no-duration"]
        cf.no_skipped_steps = arguments["--no-skipped-steps"]
        cf.with_section_names = arguments["--with-section-names"]
        cf.with_traceback = arguments["--with-traceback"]
        cf.show_metrics = arguments["--show-metrics"]

        # parse feature files
        fp = radish.FeatureParser()
        fp.parse()

        # load terrain and steps
        loader = radish.Loader(fp.get_features())
        loader.load()

        if cf.show_metrics:  # just get metrics of feature files
            ur = radish.UtilRegistry()
            if ur.has_util("show_metrics"):
                metrics = radish.Metrics(fp.get_features())
                try:
                    return ur.call_util("show_metrics", fp.get_features(), metrics.calculate())
                except KeyboardInterrupt:
                    pass
            else:
                raise radish.NoMetricUtilFoundError()
        else:  # normal run
            # run the features
            runner = radish.Runner(fp.get_features())
            endResult = runner.run()

            # report writer
            if cf.xunit_file:
                xunit = radish.XunitWriter(endResult)
                xunit.generate()

            exitCode = 0 if endResult.have_all_passed() else 1
    except radish.RadishError, e:
        e.show()
        exitCode = 2

    sys.exit(exitCode)



# -*- coding: utf-8 -*-

import os
import sys
import fnmatch
import traceback


class FileSystemHelper(object):
    @classmethod
    def expand(cls, path):
        """Expand the environment variables and the user's home directory in a given path"""
        return os.path.expanduser(os.path.expandvars(path))

    @classmethod
    def abspath(cls, path):
        """Return the absolut path of the given path"""
        return os.path.abspath(cls.expand(path))

    @classmethod
    def dirname(cls, path):
        """Return the directory name for the given path"""
        return cls.abspath(os.path.dirname(path))

    @classmethod
    def filename(cls, path, with_extension=True):
        """Return the filename of the given path"""
        f = os.path.split(path)[1]
        if not with_extension:
            f = os.path.splitext(f)[0]
        return f

    @classmethod
    def locate(cls, root, pattern):
        """Locate files in a directory recursively if it fits with the given pattern"""
        root = cls.abspath(root)
        files = []
        for p, d, f in os.walk(root):
            for filename in fnmatch.filter(f, pattern):
                files.append(os.path.join(p, filename))
        files.sort()
        return files

    @classmethod
    def import_module(cls, root, module_pattern, no_modules_error=False):
        """Import a module to the current environment"""
        files = cls.locate(root, module_pattern)
        if files:
            for f in files:
                root = cls.dirname(f)
                sys.path.insert(0, root)
                module_name = cls.filename(f, False)
                try:
                    __import__(module_name)
                    sys.path.remove(root)
                except ValueError, e:
                    if "empty module name" in traceback.format_exc(e).lower():
                        continue
                    else:
                        raise e
        elif no_modules_error:
            raise ImportError("Module '%s' in '%s' not found" % (module_pattern, root))



# -*- coding: utf-8 -*-

import sys

from radish.colorful import colorful
from radish.config import Config
from radish.hookregistry import after, before
from radish.filesystemhelper import FileSystemHelper as fsh


@before.each_feature
def print_before_feature(feature):
    if not feature.is_dry_run():
        if not Config().no_indentation:
            sys.stdout.write(feature.get_indentation())
        if not Config().no_numbers:
            sys.stdout.write(colorful.bold_white("%*d. " % (0 if Config().no_indentation else len(str(Config().highest_feature_id)), feature.get_id())))
        if Config().with_section_names:
            sys.stdout.write(colorful.bold_white("Feature: "))
        sys.stdout.write(colorful.bold_white(feature.get_sentence() + " " * (Config().longest_feature_text - len(feature.get_sentence()))))
        sys.stdout.write(" " * 10 + colorful.bold_black("# " + fsh.filename(feature.get_filename())))
        sys.stdout.write("\n")
        for l in feature.get_description().splitlines():
            if not Config().no_indentation:
                sys.stdout.write(feature.get_indentation() + " " * len(str(Config().highest_feature_id)) + "  ")
            colorful.out.white(l)
        sys.stdout.write("\n")
        sys.stdout.flush()


@before.each_scenario
def print_before_scenario(scenario):
    if not scenario.is_dry_run():
        if not Config().no_indentation:
            sys.stdout.write(scenario.get_indentation())
        if not Config().no_numbers:
            sys.stdout.write(colorful.bold_white("%*d. " % (0 if Config().no_indentation else len(str(Config().highest_scenario_id)), scenario.get_id())))
        if Config().with_section_names:
            sys.stdout.write(colorful.bold_white("Scenario: "))
        sys.stdout.write(colorful.bold_white(scenario.get_sentence()))
        sys.stdout.write("\n")
        sys.stdout.flush()


@after.each_scenario
def print_after_scenario(scenario):
    if not scenario.is_dry_run():
        sys.stdout.write("\n")
        sys.stdout.flush()


@before.each_step
def print_before_step(step):
    if not step.is_dry_run():
        if not Config().no_indentation:
            sys.stdout.write(step.get_indentation())
        if not Config().no_numbers:
            sys.stdout.write(colorful.bold_brown("%*d. " % (0 if Config().no_indentation else len(str(Config().highest_step_id)), step.get_id())))
        sys.stdout.write(colorful.bold_brown(step.get_sentence_splitted()[1]))
        sys.stdout.write("\n")
        sys.stdout.flush()


@after.each_step
def print_after_step(step):
    if not step.is_dry_run():
        splitted = step.get_sentence_splitted()
        if not Config().no_line_jump and not Config().no_overwrite:
            sys.stdout.write("\033[A\033[K" * splitted[0])

        if step.has_passed() is None and Config().no_skipped_steps:
            return

        if step.has_passed():
            color_fn = colorful.bold_green
        elif step.has_passed() is False:
            color_fn = colorful.bold_red
        elif step.has_passed() is None:
            color_fn = colorful.cyan

        if not Config().no_overwrite:
            if not Config().no_indentation:
                sys.stdout.write(step.get_indentation())
            if not Config().no_numbers:
                sys.stdout.write(color_fn("%*d. " % (0 if Config().no_indentation else len(str(Config().highest_step_id)), step.get_id())))
            sys.stdout.write(color_fn(splitted[1]))
            sys.stdout.write("\n")

        if step.has_passed() is False:
            if Config().with_traceback:
                for l in step.get_fail_reason().get_traceback().splitlines():
                    if not Config().no_indentation:
                        sys.stdout.write(step.get_sentence_indentation())
                    colorful.out.red(l)
            else:
                if not Config().no_indentation:
                    sys.stdout.write(step.get_sentence_indentation())
                print(colorful.red(step.get_fail_reason().get_name() + ": ") + colorful.bold_red(step.get_fail_reason().get_reason()))
        sys.stdout.flush()


@after.all
def print_after_all(endResult):
    if not Config().dry_run:
        white = colorful.bold_white
        green = colorful.bold_green
        red = colorful.bold_red
        cyan = colorful.cyan

        feature_text = green(str(endResult.get_passed_features()) + " passed")
        if endResult.get_failed_features() > 0:
            feature_text += white(", ") + red(str(endResult.get_failed_features()) + " failed")
        if endResult.get_skipped_features() > 0:
            feature_text += white(", ") + cyan(str(endResult.get_skipped_features()) + " skipped")

        scenario_text = green(str(endResult.get_passed_scenarios()) + " passed")
        if endResult.get_failed_scenarios() > 0:
            scenario_text += white(", ") + red(str(endResult.get_failed_scenarios()) + " failed")
        if endResult.get_skipped_scenarios() > 0:
            scenario_text += white(", ") + cyan(str(endResult.get_skipped_scenarios()) + " skipped")

        step_text = green(str(endResult.get_passed_steps()) + " passed")
        if endResult.get_failed_steps() > 0:
            step_text += white(", ") + red(str(endResult.get_failed_steps()) + " failed")
        if endResult.get_skipped_steps() > 0:
            step_text += white(", ") + cyan(str(endResult.get_skipped_steps()) + " skipped")

        colorful.out.bold_white(str(endResult.get_total_features()) + " features (%s" % (feature_text) + white(")"))
        colorful.out.bold_white(str(endResult.get_total_scenarios()) + " scenarios (%s" % (scenario_text) + white(")"))
        colorful.out.bold_white(str(endResult.get_total_steps()) + " steps (%s" % (step_text) + white(")"))

        sys.stdout.write(colorful.cyan("Run %d finished" % Config().marker))
        if not Config().no_duration:
            duration = sum([f.get_duration() for f in endResult.get_features()])
            sys.stdout.write(colorful.cyan(" within %d:%02d minutes" % (duration / 60, float(duration) % 60.0)))
        sys.stdout.write("\n")
        sys.stdout.flush()



# -*- coding: utf-8 -*-

import os

from radish.config import Config
from radish.scenario import Scenario


class Feature(object):
    def __init__(self, id, sentence, filename, line_no):
        self._id = id
        self._sentence = sentence
        self._filename = filename
        self._line_no = line_no
        self._scenarios = []
        self._description = ""

    def get_id(self):
        return self._id

    def set_id(self, id):
        self._id = id

    def get_line_no(self):
        return self._line_no

    def get_sentence(self):
        return self._sentence

    def set_sentence(self, sentence):
        self._sentence = sentence

    def get_filename(self):
        return self._filename

    def get_description(self):
        return self._description

    def get_indentation(self):
        return "  "

    def get_duration(self):
        d = 0
        for s in self._scenarios:
            d += s.get_duration()
        return d

    def is_dry_run(self):
        return Config().dry_run

    def get_scenarios(self):
        return self._scenarios

    def get_scenario(self, id):
        if id == -1:
            return self._scenarios[-1]

        for s in self._scenarios:
            if s.get_id() == id:
                return s
        return None

    def has_passed(self):
        skipped = True
        for s in self._scenarios:
            if s.has_passed() is False:
                return False
            elif s.has_passed():
                skipped = False
        return None if skipped else True

    def append_scenario(self, scenario):
        if isinstance(scenario, Scenario):
            self._scenarios.append(scenario)

    def append_description_line(self, line):
        if self._description == "":
            self._description = line
        else:
            self._description += os.linesep + line



# -*- coding: utf-8 -*-

import sys
import traceback

from radish.config import Config
from radish.colorful import colorful
from radish.hookregistry import HookRegistry
from radish.endresult import EndResult


class Runner(object):
    def __init__(self, features):
        self._features = features

    def run(self):
        abort = False
        interrupted = False

        hr = HookRegistry()
        e = hr.call_hook("before", "all")
        if e is not None:
            self._print_traceback(e)
            abort = True

        if not abort:
            for f in self._features:
                e = hr.call_hook("before", "feature", f)
                if e is not None:
                    self._print_traceback(e)
                    return self._create_endresult()

                for s in f.get_scenarios():
                    e = hr.call_hook("before", "scenario", s)
                    if e is not None:
                        self._print_traceback(e)
                        return self._create_endresult()
                    skip_remaining_steps = False

                    for step in s.get_steps():
                        e = hr.call_hook("before", "step", step)
                        if e is not None:
                            self._print_traceback(e)
                            return self._create_endresult()

                        if not skip_remaining_steps and not interrupted:
                            try:
                                passed = step.run()
                                if not passed and not Config().dry_run:
                                    skip_remaining_steps = True
                                    if Config().abort_fail:
                                        abort = True
                            except KeyboardInterrupt:
                                interrupted = True
                                sys.stdout.write("\r")

                        e = hr.call_hook("after", "step", step)
                        if e is not None:
                            self._print_traceback(e)
                            return self._create_endresult()
                    e = hr.call_hook("after", "scenario", s)
                    if e is not None:
                        self._print_traceback(e)
                        return self._create_endresult()
                    if abort:  # if -a is set
                        break
                e = hr.call_hook("after", "feature", f)
                if e is not None:
                    self._print_traceback(e)
                    return self._create_endresult()
                if abort:  # if -a is set
                    break
        return self._create_endresult()

    def _print_traceback(self, fail_reason):
        print(colorful.bold_red(fail_reason.get_name()) + colorful.red(" exception caught from external hook at ") + colorful.bold_red(fail_reason.get_filename()) + colorful.red(":") + colorful.bold_red(fail_reason.get_line_no()))
        if Config().with_traceback:
            for l in fail_reason.get_traceback().splitlines():
                colorful.out.red("  " + l)
        else:
            print("  " + colorful.red(fail_reason.get_name() + ": ") + colorful.bold_red(fail_reason.get_reason()))
        sys.stdout.write("\n")

    def _create_endresult(self):
        endResult = EndResult(self._features)
        e = HookRegistry().call_hook("after", "all", endResult)
        if e is not None:
            self._print_traceback(e)
        return endResult



# -*- coding: utf-8 -*-

import traceback
import inspect
import sys
import datetime
import re

from radish.config import Config
from radish.utilregistry import UtilRegistry
from radish.exceptions import ValidationError, RadishError


class Step(object):
    CHARS_PER_LINE = 100

    def __init__(self, id, scenario, sentence, filename, line_no):
        self._id = id
        self._scenario = scenario
        self._sentence = sentence
        self._filename = filename
        self._line_no = line_no
        self._function = None
        self._metric_indicators = None
        self._match = None
        self._passed = None
        self._fail_reason = None
        self._starttime = None
        self._endtime = None
        self._validation_error = False

    def get_id(self):
        return self._id

    def get_scenario(self):
        return self._scenario

    def get_line_no(self):
        return self._line_no

    def get_sentence(self):
        return self._sentence

    def get_sentence_splitted(self):
        ur = UtilRegistry()
        if ur.has_util("split_sentence"):
            try:
                return ur.call_util("split_sentence", self)
            except KeyboardInterrupt:
                pass
        splitted = [self._sentence[i:i + Step.CHARS_PER_LINE] for i in range(0, len(self._sentence), Step.CHARS_PER_LINE)]
        if Config().no_indentation:
            return len(splitted), "\n".join(splitted)
        return len(splitted), ("\n" + self.get_sentence_indentation()).join(splitted)

    def get_filename(self):
        return self._filename

    def get_indentation(self):
        return "  " + " " * (len(str(Config().highest_feature_id)) + len(str(Config().highest_scenario_id))) + "    "

    def get_sentence_indentation(self):
        return self.get_indentation() + " " * len(str(Config().highest_step_id)) + "  "

    def is_dry_run(self):
        return Config().dry_run

    def get_function(self):
        return self._function

    def set_function(self, function):
        self._function = function

    def get_metric_indicators(self):
        return self._metric_indicators

    def set_metric_indicators(self, metric_indicators):
        self._metric_indicators = metric_indicators

    def get_match(self):
        return self._match

    def set_match(self, match):
        self._match = match

    def has_passed(self):
        return self._passed

    def get_duration(self):
        if self._passed is True or self._passed is False:
            td = self._endtime - self._starttime
            return (td.microseconds + (td.seconds + td.days * 24 * 3600) * 1e6) / 1e6
        return 0

    def get_fail_reason(self):
        return self._fail_reason

    class FailReason(object):
        def __init__(self, e):
            self._exception = e
            self._reason = unicode(str(e), "utf-8")
            self._traceback = traceback.format_exc()
            self._name = e.__class__.__name__
            tr_infos = traceback.extract_tb(sys.exc_info()[2])[-1]
            self._filename = tr_infos[0]
            self._line_no = tr_infos[1]

        def get_reason(self):
            return self._reason

        def get_traceback(self):
            return self._traceback

        def get_name(self):
            return self._name

        def get_filename(self):
            return self._filename

        def get_line_no(self):
            return self._line_no

    def run(self):
        kw = self._match.groupdict()
        try:
            self._starttime = datetime.datetime.now()
            if kw:
                self._function(self, **kw)
            else:
                self._function(self, *self._match.groups())
            self._passed = not self._validation_error
        except KeyboardInterrupt:
            raise
        except Exception, e:
            self._passed = False
            self._fail_reason = Step.FailReason(e)
            if self.is_dry_run():
                caller = inspect.trace()[-1]
                sys.stderr.write("%s:%d: error: %s\n" % (caller[1], caller[2], self._fail_reason.get_reason().encode("utf-8")))
        self._endtime = datetime.datetime.now()
        return self._passed

    def ValidationError(self, msg):
        self._validation_error = True
        if self.is_dry_run():
            sys.stderr.write("%s:%d: error: %s\n" % (self._filename, self._line_no, msg))
        else:
            raise ValidationError(msg)

    def get_report_as_xunit_tag(self):
        try:
            from lxml import etree
        except:
            raise RadishError("No lxml support. Please install python-lxml")

        testcase = etree.Element(
            "testcase",
            id="%d.%d.%d" % (self._scenario.get_feature().get_id(), self._scenario.get_id(), self._id),
            classname="%s/%s" % (self._scenario.get_feature().get_sentence().replace("/", "\/"), self._scenario.get_sentence().replace("/", "\/")),
            name=self._sentence,
            time=str(self.get_duration())
        )
        if self._passed is False:
            failure = etree.Element(
                "failure",
                type=self._fail_reason.get_name(),
                message=self._strip_ansi_text(self._fail_reason.get_reason())
            )
            failure.text = etree.CDATA(self._strip_ansi_text(self._fail_reason.get_traceback()))
            testcase.append(failure)
        elif self._passed is None:
            skipped = etree.Element(
                "skipped",
                type="NormalSkip"
            )
            testcase.append(skipped)
        return testcase

    # FIXME: register this methods somewhere as util
    def _strip_ansi_text(self, text):
        pattern = re.compile("(\\033\[\d+(?:;\d+)*m)")
        return pattern.sub("", text)



# -*- coding: utf-8 -*-

import os
import re
import pickle
import codecs

from radish.config import Config
from radish.feature import Feature
from radish.scenario import Scenario
from radish.step import Step
from radish.filesystemhelper import FileSystemHelper as fsh
from radish.exceptions import RadishError, FeatureFileNotFoundError, FeatureFileNotValidError


class FeatureParser(object):
    def __init__(self):
        self._features = []
        self._feature_files = []
        for f in Config().feature_files:
            if os.path.isdir(f):
                self._feature_files.extend(fsh.locate(f, "*.feature"))
            else:
                self._feature_files.append(f)
        self._comment_pattern = re.compile("^[\s]*?#")
        self._feature_pattern = re.compile("Feature: ?(.*)$")
        self._scenario_pattern = re.compile("Scenario: ?(.*)$")
        self._loop_modifier_pattern = re.compile("run (\d+) times")

    def get_features(self):
        return self._features

    def parse(self):
        conf = Config()
        conf.highest_feature_id = 0
        conf.highest_scenario_id = 0
        conf.highest_step_id = 0
        conf.longest_feature_text = 0
        self._feature_id = 0
        for f in self._feature_files:
            try:
                self._features.extend(self._parse_feature(f))
            except RadishError:
                raise
            except:
                raise FeatureFileNotValidError(f)
        conf.highest_feature_id = self._feature_id

    def _parse_feature(self, feature_file):
        if not os.path.exists(feature_file):
            raise FeatureFileNotFoundError(feature_file)

        features = []
        in_feature = False
        feature_loop = None
        scenario_id = 0
        scenario_loop = None
        step_id = 0
        line_no = 0

        f = codecs.open(feature_file, "r", "utf-8")
        for l in f.readlines():
            line_no += 1
            if not l.strip() or self._comment_pattern.search(l):
                continue

            feature_match = self._feature_pattern.search(l)
            scenario_match = self._scenario_pattern.search(l)

            if feature_match:  # create new feature
                if scenario_loop:
                    self._repeat_scenario(scenario_loop[1], scenario_loop[2], features, scenario_id)
                scenario_loop = None

                if feature_loop:
                    self._repeat_feature(feature_loop[1], feature_loop[2], features)
                feature_loop = None

                in_feature = True
                self._feature_id += 1
                scenario_id = 0

                sentence, modifiers = self._get_sentence_modifiers(feature_match.group(1))
                feature_loop = modifiers["loop"]

                features.append(Feature(self._feature_id, sentence, feature_file, line_no))
                if len(feature_match.group(1)) > Config().longest_feature_text:
                    Config().longest_feature_text = len(feature_match.group(1))
            elif scenario_match:  # create new scenario
                if scenario_loop:
                    scenario_id = self._repeat_scenario(scenario_loop[1], scenario_loop[2], features, scenario_id)
                scenario_loop = None

                in_feature = False
                scenario_id += 1
                step_id = 0

                sentence, modifiers = self._get_sentence_modifiers(scenario_match.group(1))
                scenario_loop = modifiers["loop"]

                features[-1].append_scenario(Scenario(scenario_id, features[-1], sentence, feature_file, line_no))
                if scenario_id > Config().highest_scenario_id:
                    Config().highest_scenario_id = scenario_id
            else:  # create new step or append feature description line
                line = l.rstrip(os.linesep).strip()
                if not in_feature:
                    step_id += 1
                    features[-1].get_scenarios()[-1].append_step(Step(step_id, features[-1].get_scenario(-1), line, feature_file, line_no))
                    if step_id > Config().highest_step_id:
                        Config().highest_step_id = step_id
                else:
                    features[-1].append_description_line(line)
                    if len(line) + 2 > Config().longest_feature_text:
                        Config().longest_feature_text = len(line) + 2

        if scenario_loop:
            self._repeat_scenario(scenario_loop[1], scenario_loop[2], features, scenario_id)

        if feature_loop:
            self._repeat_feature(feature_loop[1], feature_loop[2], features)

        f.close()
        return features

    def _get_sentence_modifiers(self, sentence):
        orig_sentence = sentence
        sentence_parts = [p.strip() for p in orig_sentence.split("|")]
        sentence = sentence_parts.pop(0)
        modifiers = {"loop": None}
        while sentence_parts:
            p = sentence_parts.pop(0)
            loop_match = self._loop_modifier_pattern.search(p)
            if loop_match:
                modifiers["loop"] = (True, int(loop_match.group(1)) - 1, sentence)
        if modifiers["loop"]:
            sentence += " | run 1 of %d" % (modifiers["loop"][1] + 1)
        else:
            sentence = orig_sentence
        return sentence, modifiers

    def _repeat_feature(self, times, sentence, features):
        feature = features[-1]
        pickle.dump(feature, open(".repeated_feature.p", "wb"))
        for i in range(times):
            self._feature_id += 1
            new_sentence = sentence + " | run %d of %d" % (i + 2, times + 1)
            new_feature = pickle.load(open(".repeated_feature.p", "rb"))
            new_feature.set_id(self._feature_id)
            new_feature.set_sentence(new_sentence)
            features.append(new_feature)
            if len(new_sentence) > Config().longest_feature_text:
                Config().longest_feature_text = len(new_sentence)
        os.remove(".repeated_feature.p")

    def _repeat_scenario(self, times, sentence, features, last_scenario_id):
        scenario = features[-1].get_scenario(last_scenario_id)
        pickle.dump(scenario, open(".repeated_scenario.p", "wb"))
        for i in range(times):
            last_scenario_id += 1
            new_scenario = pickle.load(open(".repeated_scenario.p", "rb"))
            new_scenario.set_id(last_scenario_id)
            new_scenario.set_sentence(sentence + " | run %d of %d" % (i + 2, times + 1))
            features[-1].append_scenario(new_scenario)
            if last_scenario_id > Config().highest_scenario_id:
                Config().highest_scenario_id = last_scenario_id
        os.remove(".repeated_scenario.p")
        return last_scenario_id



# -*- coding: utf-8 -*-

import thread


def singleton(multi_init=False):
    def _singleton_decorator(decorated_class):
        """decorator for a thread-safe singleton pattern.
            Please just use the singleton pattern if it is not
            possible do solve your problem in another way.

            Example:

            >>> @singleton()
            ... class MySingleton(object):
            ...     pass
            >>> o = MySingleton()
            >>> o2 = MySingleton()
            >>> print o is o2

        """
        class _singleton(decorated_class):
            _instance = None
            _lock = thread.allocate_lock()

            def __new__(cls, *args, **kwargs):
                with cls._lock:
                    if _singleton._instance is None:
                        _singleton._instance = super(_singleton, cls).__new__(cls, *args, **kwargs)
                        _singleton._instance._initialized = False
                return _singleton._instance

            def __init__(self, *args, **kwargs):
                if self._initialized and not multi_init:
                    return
                super(_singleton, self).__init__(*args, **kwargs)
                self._initialized = True

        _singleton.__name__ = decorated_class.__name__
        return _singleton
    return _singleton_decorator



# -*- coding: utf-8 -*-

import os
from datetime import datetime

from radish.config import Config
from radish.filesystemhelper import FileSystemHelper as fsh
from radish.exceptions import RadishError


class XunitWriter(object):
    REPORT_FILENAME = "radishtests.xml"
    ONE_XUNIT = "radish.one_xunit"

    def __init__(self, endResult):
        self._endResult = endResult

    def generate(self):
        try:
            from lxml import etree
        except:
            raise RadishError("No lxml support. Please install python-lxml")

        outputs = {}
        if Config().split_xunit:
            for f in self._endResult.get_features():
                filename = fsh.filename(f.get_filename(), with_extension=False)
                path = os.path.join(Config().xunit_file, filename + ".xml")
                if path not in outputs:
                    outputs[path] = []
                outputs[path].append(f)
        else:
            outputs[XunitWriter.ONE_XUNIT] = self._endResult.get_features()

        for filename, features in outputs.iteritems():
            if filename == XunitWriter.ONE_XUNIT:
                filename = Config().xunit_file or XunitWriter.REPORT_FILENAME

            testsuite = etree.Element(
                "testsuite",
                name="radish",
                hostname="localhost",
                id=unicode(Config().marker),
                time=str(sum([f.get_duration() for f in features])),
                tests=str(self._endResult.get_total_steps()),
                failures=str(self._endResult.get_failed_steps()),
                skipped=str(self._endResult.get_skipped_steps()),
                errors="0",
                timestamp=datetime.now().strftime("%Y-%m-%dT%H:%M:%S")
            )

            # append steps to testsuite
            for f in features:
                for s in f.get_scenarios():
                    for step in s.get_steps():
                        testsuite.append(step.get_report_as_xunit_tag())

            with open(filename, "w") as f:
                f.write(etree.tostring(testsuite, pretty_print=True, xml_declaration=True, encoding="utf-8"))



# -*- coding: utf-8 -*-

import re

from radish.singleton import singleton


@singleton()
class StepRegistry(object):
    def __init__(self):
        self._steps = {}

    def register(self, regex, func, metric_indicators):
        self._steps[regex] = {"func": func, "metric_indicators": metric_indicators}

    def find(self, sentence):
        for regex, data in self._steps.items():
            match = re.search(regex, sentence)
            if match:
                return match, data["func"], data["metric_indicators"]
        return None, None, None



# -*- coding: utf-8 -*-


class EndResult(object):
    def __init__(self, features):
        self._features = features
        self._total_features = 0
        self._total_scenarios = 0
        self._total_steps = 0

        self._passed_features = 0
        self._failed_features = 0
        self._skipped_features = 0

        self._passed_scenarios = 0
        self._failed_scenarios = 0
        self._skipped_scenarios = 0

        self._passed_steps = 0
        self._failed_steps = 0
        self._skipped_steps = 0

        self._total_features = len(features)
        for f in features:
            feature_outcome = f.has_passed()
            if feature_outcome:
                self._passed_features += 1
            elif feature_outcome is False:
                self._failed_features += 1
            elif feature_outcome is None:
                self._skipped_features += 1

            self._total_scenarios += len(f.get_scenarios())

            for s in f.get_scenarios():
                scenario_outcome = s.has_passed()
                if scenario_outcome:
                    self._passed_scenarios += 1
                elif scenario_outcome is False:
                    self._failed_scenarios += 1
                elif scenario_outcome is None:
                    self._skipped_scenarios += 1

                self._total_steps += len(s.get_steps())

                for step in s.get_steps():
                    step_outcome = step.has_passed()
                    if step_outcome:
                        self._passed_steps += 1
                    elif step_outcome is False:
                        self._failed_steps += 1
                    elif step_outcome is None:
                        self._skipped_steps += 1

    def have_all_passed(self):
        return self._total_features == self._passed_features

    def get_total_features(self):
        return self._total_features

    def get_features(self):
        return self._features

    def get_passed_features(self):
        return self._passed_features

    def get_failed_features(self):
        return self._failed_features

    def get_skipped_features(self):
        return self._skipped_features

    def get_total_scenarios(self):
        return self._total_scenarios

    def get_passed_scenarios(self):
        return self._passed_scenarios

    def get_failed_scenarios(self):
        return self._failed_scenarios

    def get_skipped_scenarios(self):
        return self._skipped_scenarios

    def get_total_steps(self):
        return self._total_steps

    def get_passed_steps(self):
        return self._passed_steps

    def get_failed_steps(self):
        return self._failed_steps

    def get_skipped_steps(self):
        return self._skipped_steps



# -*- coding: utf-8 -*-

from radish.singleton import singleton
from radish.config import *
from radish.featureparser import *
from radish.feature import *
from radish.scenario import *
from radish.step import *
from radish.runner import *
from radish.loader import *
from radish.filesystemhelper import *
from radish.stepdecorator import *
from radish.stepregistry import *
from radish.hookregistry import *
from radish.utilregistry import *
from radish.terrain import *
from radish.colorful import *
from radish.endresult import *
from radish.exceptions import *
from radish.xunitwriter import *
from radish.metrics import *
from radish.writer import *
from radish.version import *



# -*- coding: utf-8 -*-

from radish.config import Config
from radish.stepregistry import StepRegistry
from radish.filesystemhelper import FileSystemHelper as fsh
from radish.exceptions import StepDefinitionFileNotFoundError, StepDefinitionNotFoundError


class Loader(object):
    def __init__(self, features):
        self._features = features

    def load(self):
        self._load_terrain()
        self._load_step_definitions()
        self._load_logger()

    def _load_terrain(self):
        fsh.import_module(Config().basedir, "terrain.py")

    def _load_step_definitions(self):
        if not fsh.locate(Config().basedir, "steps.py"):
            raise StepDefinitionFileNotFoundError(Config().basedir, "steps.py")

        fsh.import_module(Config().basedir, "steps.py")
        self._merge_steps_with_definitions()

    def _load_logger(self):
        if fsh.locate(Config().basedir, "logger.py"):
            fsh.import_module(Config().basedir, "logger.py")

    def _merge_steps_with_definitions(self):
        sr = StepRegistry()
        for feature in self._features:
            for scenario in feature.get_scenarios():
                for step in scenario.get_steps():
                    match, func, metric_indicators = sr.find(step.get_sentence())
                    if match and func:
                        step.set_function(func)
                        step.set_match(match)
                        step.set_metric_indicators(metric_indicators)
                    else:
                        raise StepDefinitionNotFoundError(step.get_sentence(), step.get_filename(), step.get_line_no())



#!/usr/bin/python
# -*- coding: utf-8 -*-

from imp import load_source
from setuptools import setup

radish_version = load_source("version", "radish/version.py")

setup(
    name="radish-bdd",
    version=radish_version.__version__,
    license="GPL",
    description="Behaviour-Driven-Development tool for python",
    author="Timo Furrer",
    author_email="tuxtimo@gmail.com",
    maintainer="Timo Furrer",
    maintainer_email="tuxtimo@gmail.com",
    platforms=["Linux", "Windows", "MAC OS X"],
    url="http://github.com/timofurrer/radish",
    download_url="http://github.com/timofurrer/radish",
    #install_requires=["docopt==0.6.1", "lxml==3.2.1"],
    install_requires=["docopt==0.6.1"],
    package_dir={"radish.singleton": "radish/pysingleton/singleton"},
    packages=["radish", "radish.singleton"],
    data_files=[("templates", ["templates/steps.py", "templates/terrain.py"])],
    entry_points={"console_scripts": ["radish = radish.main:main"]},
    package_data={"radish": ["*.md"]},
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Intended Audience :: Education",
        "Intended Audience :: Other Audience",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Natural Language :: English",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: OS Independent",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: Microsoft :: Windows :: Windows 7",
        "Operating System :: Microsoft :: Windows :: Windows XP",
        "Operating System :: POSIX",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: Implementation",
        "Topic :: Education :: Testing",
        "Topic :: Software Development",
        "Topic :: Software Development :: Testing"
    ],
)



# -*- coding: utf-8 -*-

import re

from radish.stepregistry import StepRegistry
from radish.exceptions import StepLoadingError


def step(regex, metric_indicators=None):
    def wrapper(func):
        try:
            re.compile(regex)
        except Exception:
            raise StepLoadingError(regex)
        StepRegistry().register(regex, func, metric_indicators)
        return func
    return wrapper



# -*- coding: utf-8 -*-

import os

from radish.exceptions import BasedirNotFoundError


class Config(object):
    _instance = None

    class __Config(object):
        def SetBasedir(self, basedir):
            if not os.path.exists(basedir) or not os.path.isdir(basedir):
                raise BasedirNotFoundError(basedir)
            self.basedir = basedir

    def __new__(cls):
        if not Config._instance:
            Config._instance = Config.__Config()
        return Config._instance

    def __getattr__(self, key):
        return getattr(self._instance, key)

    def __setattr__(self, key, value):
        return setattr(self._instance, key, value)
